#!/usr/bin/perl -w  # this path to Perl is your_unix_system-dependent: either /usr/bin/perl or /usr/local/bin/perl
# Author:  Dr. Ruming Li -- January 16, 2011
# Purpose: Integrate gene info from different sources and generate biological and replicate oligo groups
# Usage:   perl bio-rep-maker.pl input_file [output_file1][output_file2][output_file3][all_other_options]
#--------------------------------------------------------------------------------------------------------
use strict;
use warnings;
$0=~s/^\.\///;
my $usage = "\nUsage:\n\n$0 [switches]\n\nSwitches:\n\n-m <input-files-mode>\n-f <input-genbank-file>
-o <input-oligos-file>\n-b <output-bio-oligos>\n-r <output-rep-oligos>\n-d <output-data-info>
-s <output-statistics>\n-i <ignore-gene-entries>\n-n <number-rep-oligos>\n-L <length-org-initial>
-p <skip-pseudo-genes>\n-R <ignore-RNA-entries>\n-g <unique-gene-sb-check>\n-l <locus-identity-check>
-v <sorted-gbk-data-view>\n-h <program-usage-info>\n
Note: the spacing following a switch (-x) is optional; eg, either '-i1' or '-i 1' is acceptable.


Synopsis:


-m <input-files-mode>

   The files-input-mode refers to genbank files and has 3 options:

   1--input a single/multiple file(s) by entering filename(s), separated by the comma (,) without
      spaces and input in the order listed,

   2--input many files by a file in which filenames are listed with one name a line, and

   3--get the group of files stored in a directory and input them by entering the path to that
      directory.

-f <input-genbank-file>

   are files from NCBI databases (eg, GenBank, Genome, Taxonomy, RefSeq, ftp://..., etc.) in which
   all entries of data, nomenclature, notations, and formats are treated as official or established
   (although not really well-curated information/resource/collection). As such, they serve as the
   standards (norms) for data integration since all of their data (mainly for gene symbols and gene
   loci/coordinates) are one-contig-based and hence referenceable. In addition, these data are more
   accessible and browsable. Therefore, they are collected and input to make up data deficiencies
   (mainly for missing gene symbols and loci/coordinates) and achieve a better data integrity (ie,
   for incorporation of gene IDs into data pool).

-o <input-oligos-file>

   is the oligo containing file output from running the program oligo-maker-v.2.0.pl. This file is
   better off being renamed for the recognizable purposes such as 'genome-name-oligos.tdt'.

-b <output-bio-oligos>

   is one of the destination files that contain all oligos designed and constructed as well as other
   gene features required for a web submission/uploading. This file is used to create a non-control
   or biological probe group and does not include the oligo entries from a replicate probe group.

-r <output-rep-oligos>

   is one of the destination files that contain at least 300 oligos designed and randomly constructed
   as well as other gene features required for a web submission/uploading. This file is used to
   create a replicate probe group and all of its entries are not included in a biological probe group.

-d <output-data-info>

   is the file containing gene/oligo tagging records for data research and user information.

-s <output-statistics>

   is the file containing a summary of statistics for sequence data subjected to processing.

-i <ignore-gene-entries>

   refers to a filter that gives protein keywords/full-names on which ignoring genes will be based.
   The keywords/full-names in the list are delimted by a pipe (|) without spaces.

-n <number-rep-oligos>

   refers to the size of a replicate probe group to be created. The default is 300 oligo probes.

-L <length-org-initial>

   defines the length of organism's initial. The default is 2 characters long (ie, the 1st letter of
   genus name + the 1st letter of species name) and the value zero (0) removes it from appending to
   the gene locus.

-p <skip-pseudo-genes>

   provides the 0-1 options to treat pseudogenes. The default (1) does not take them into account.

-R <ignore-RNA-entries>

   refers to a filter whose option (0-1) treats the way of inclusion of RNAs. The default (1) does
   not take them into account.

-g <unique-gene-sb-check>

   refers to additional data inspection whose option (1) detects duplicate gene symbols for the
   first time and (0) disables this feature.

-l <locus-identity-check>

   refers to additional data inspection whose option (1) checks the equality of loci for the first
   time and (0) disables it.

-v <sorted-gbk-data-view>

   refers to an optional data view whose value (0-1) toggles a listing of extracted GenBank data
   sorted by gene symbol.\n";
my ($e,$i,$j,$k,$l,$m,$n,@psd,@prd,@seq,@idx,@tag,@acn,@gid,@oid,@gsb,@olg,@loc,@prt,@rnd);
my (@a,@b,@c,@d,@s,@ACN,@GSB,@GID,@pGI,@LOC);

# Defaults
my $gbkFile="input-genes.gbk";
my $olgFile="input-oligo.tdt";
my $outBios="biological_probe_group.tdt";
my $outReps="replicate_probe_group.tdt";
my $outData="out-data-info.tdt";
my $outStat="out-statistics.tdt";
my $ignItem="hypothetical|putative|pseudo";
my $numReps=300;
my $lenInit=2;
my $skiPseu=1;
my $skipRNA=1;
my $gsCheck=0;
my $lcCheck=0;
my $gbkSort=0;
my $inpMode=3;

unless (@ARGV) {}
else {while ($_=shift) {
if (/^-[LRbdflgrimvphosn]/) {
if ($_ eq "-f") {$gbkFile=(defined(local $_=shift)?$_:"");} elsif (/^-f/) {($gbkFile)=/-f(.+)/;}
if ($_ eq "-o") {$olgFile=(defined(local $_=shift)?$_:"");} elsif (/^-o/) {($olgFile)=/-o(.+)/;}
if ($_ eq "-b") {$outBios=(defined(local $_=shift)?$_:"");} elsif (/^-b/) {($outBios)=/-b(.+)/;}
if ($_ eq "-r") {$outReps=(defined(local $_=shift)?$_:"");} elsif (/^-r/) {($outReps)=/-r(.+)/;}
if ($_ eq "-d") {$outData=(defined(local $_=shift)?$_:"");} elsif (/^-d/) {($outData)=/-d(.+)/;}
if ($_ eq "-s") {$outStat=(defined(local $_=shift)?$_:"");} elsif (/^-s/) {($outStat)=/-s(.+)/;}
if ($_ eq "-i") {$ignItem=(defined(local $_=shift)?$_:"");} elsif (/^-i/) {($ignItem)=/-i(.+)/;}
if ($_ eq "-n") {$numReps=(defined(local $_=shift)?$_:0);} elsif (/^-n/) {($numReps)=/-n(.+)/;}
if ($_ eq "-L") {$lenInit=(defined(local $_=shift)?$_:0);} elsif (/^-L/) {($lenInit)=/-L(.+)/;}
if ($_ eq "-p") {$skiPseu=(defined(local $_=shift)?$_:0);} elsif (/^-p/) {($skiPseu)=/-p(.+)/;}
if ($_ eq "-R") {$skipRNA=(defined(local $_=shift)?$_:0);} elsif (/^-R/) {($skipRNA)=/-R(.+)/;}
if ($_ eq "-g") {$gsCheck=(defined(local $_=shift)?$_:0);} elsif (/^-g/) {($gsCheck)=/-g(.+)/;}
if ($_ eq "-l") {$lcCheck=(defined(local $_=shift)?$_:0);} elsif (/^-l/) {($lcCheck)=/-l(.+)/;}
if ($_ eq "-v") {$gbkSort=(defined(local $_=shift)?$_:0);} elsif (/^-v/) {($gbkSort)=/-v(.+)/;}
if ($_ eq "-m") {$inpMode=(defined(local $_=shift)?$_:0);} elsif (/^-m/) {($inpMode)=/-m(.+)/;}
if ($_ eq "-h") {print "$usage\n";exit;}
} else {die "\nUnknown argument/switch $_\n$usage\n";}}}
if ($ignItem=~/\s/) {die "$ignItem: Invalid entries! No spaces between them!\n";}
if ($gbkFile!~/^[\w\/.,-]*\w$/) {die "$gbkFile: Invalid file name!\n";}
if ($olgFile!~/^[\w\/.-]*\w$/) {die "$olgFile: Invalid file name!\n";}
if ($numReps!~/^\d+$/) {die "$numReps: Not numerical parameter!\n";}
if ($lenInit!~/^\d+$/) {die "$lenInit: Not numerical parameter!\n";}
if ($skiPseu!~/^[01]$/) {die "$skiPseu: Not a 0-1 number!\n";}
if ($skipRNA!~/^[01]$/) {die "$skipRNA: Not a 0-1 number!\n";}
if ($gsCheck!~/^[01]$/) {die "$gsCheck: Not a 0-1 number!\n";}
if ($lcCheck!~/^[01]$/) {die "$lcCheck: Not a 0-1 number!\n";}
if ($gbkSort!~/^[01]$/) {die "$gbkSort: Not a 0-1 number!\n";}
if ($inpMode!~/^[123]$/) {die "$inpMode: Not a 1-3 number!\n";}

print "\nThis program will run based on the following settings:\n
-m files_input_mode    = $inpMode
-f input_genbank_file  = $gbkFile
-o input_oligos_file   = $olgFile
-b output_bio_oligos   = $outBios
-r output_rep_oligos   = $outReps
-d output_data_info    = $outData
-s output_statistics   = $outStat
-i ignore_gene_entries = $ignItem
-n number_rep_oligos   = $numReps
-L length_org_initial  = $lenInit
-p skip_pseudo_genes   = $skiPseu
-R ignore_RNA_entries  = $skipRNA
-g unique_genesb_check = $gsCheck
-l loci_equality_check = $lcCheck
-v sort_gbk_data_view  = $gbkSort\n
Type in <$0 -h> for help and make your own settings.\n\n";

do {
	print "Proceed to run ? [y/n]: ";
	$_ = <STDIN>;
	if (/^n$/i) {exit;}
} until (/^y$/i);
print "\n";

if ($inpMode==1) {
	if (-d $gbkFile) {
		die "$gbkFile: This is a directory, not file(s).\n";
	}
} elsif ($inpMode==2) {
	unless (-f $gbkFile) {die "$gbkFile: Not a file.\n";}
	open(GBKFILE, "<$gbkFile") or die "$gbkFile: No such a file.\n";
	@acn=<GBKFILE>;
	$gbkFile=join('',@acn);
	$gbkFile=~s/\n/,/g;
	close(GBKFILE);
} elsif ($inpMode==3) {
	opendir(dH, $gbkFile) || die "can't open directory $gbkFile: $!";
	# get all file entries in the directory w/o dots (.) and (..)
	@acn=grep(!/^\./, readdir(dH));
	@acn=sort(@acn);
	$s[0]="$gbkFile/"; # save dir path
	$gbkFile=join(',',@acn);
	closedir(dH);
}

# Read in GenBank data by single/multiple file input
$l=$m=$n=0;
$s[1]="";
while ($gbkFile=~/([\w\/.-]*\w)/g) {
	print "Processing $1...\n";
	open (GBKFILE, "<".(($inpMode==3)?$s[0].$1:$1)) or die "$1: No such a file.\n";
while (<GBKFILE>) {
	if (/^VERSION\s+(\S*)\s+GI:\s*(\S*)/) { # Grab primary accession number and GI
	push(@a,$1);
	push(@b,$2);
	last;
}}
if ($s[1] eq "") {
while (<GBKFILE>) {
	if (/^\s+ORGANISM\s+(\w)(\S*)\s+(\S*)/) { # Grab organism name
	$s[1]="$1$2 $3"; # genus name + species name
	$s[2]=lc($1);    # genus initial
	$s[3]=length($3);
	if ($lenInit <= $s[3]) {$s[3]=$lenInit?$lenInit-1:0;}
	$3 =~ /^(\w{$s[3]})/;
	$s[2] .= $1; # + species initial
	if ($lenInit) {
		$s[1] .= " ($s[2])";
		$s[2] .= "|";
	} else {
		$s[2]="";
	}
	last;
}}}
$s[4]="";
while (($_=<GBKFILE>)!~/^ORIGIN/) {
	if (/^\s+gene\s{3,}(\S*)/) { # match gene and grab locus
		# translate "complement()" into minus strand and get array element subscript index
		$e=push(@LOC,(($1=~/complement\((\S+\.\.\S+)\)/)?"$1(-)":"$1(+)"))-1;
		$GID[$e]=$b[$#b]; # initialize GID array w/ GI if no GeneID
		$psd[$e]=""; # initialize pseudo array if no pseudo
		$GSB[$e]=$prd[$e]=$pGI[$e]="";
		$ACN[$e]=$a[$#a];
		$i=1; # control redundant locus_tag matches
		$j=0; # control toggle
	} elsif ($i&&/\/locus_tag="\s*(\S*)\s*"/) { # Grab gene symbol
		if ($GSB[$e] eq "") {
			$GSB[$e]=$1; # $1 should be used before match statement below
			if ($s[4] eq "") {push(@c,$1);$s[4]='1';} # save 1st g-symbol
			if ($1=~/_R/) {$n++;} # count RNAs
		}
		if ($j) {$i=0;}
	} elsif (/\/(pseudo)/i) { # Grab pseudogene if it exists
		$psd[$e]=$1;
		$m++; # count pseudogenes
	} elsif (/="GeneID:\s*(\S*)\s*"/) { # Grab NCBI gene identifier (gi)
		if ($1!~/^\s*$/ && $1!~/$GID[$e]/) {$GID[$e]=$1;}
	} elsif (/^\s+CDS\s{3,}\S*/) { # match coding sequence
		$l++; # count CDSs
		$j=1;
	} elsif (/\/product="(.*)"/) { # Grab gene product name
		$prd[$e]=$1;
	} elsif (/\/product="([^"]*[^"\s])$/) { # catch dangling long product name
		$prd[$e]=$1;
		$k=1;
	} elsif ($k&&/^\s+(\s.*)"/) { # join partial product names
		$prd[$e].=$1;
		$k=0;
	} elsif (/="GI:\s*(\S*)\s*"/) { # Grab protein genbank identifier
		$pGI[$e]=$1;
	}
}
if ($s[4] eq '1') {
	push(@d,$GSB[$e]); # save last gene symbol
} else {
	pop(@a);
	pop(@b);
}
close(GBKFILE);
}

open (OLGFILE, "<$olgFile") or die "$olgFile: No such a file.\n";
open (BIOFILE, ">$outBios") or die "$!";
open (REPFILE, ">$outReps") or die "$!";
open (OUTDATA, ">$outData") or die "$!";
open (OUTSTAT, ">$outStat") or die "$!";

print OUTSTAT "\nOrganism: $s[1]\n";
print "\nOrganism: $s[1]\n";
print OUTSTAT "Number of GenBank genes = ",scalar @LOC,"\n";
print "Number of GenBank genes = ",scalar @LOC,"\n";
print OUTSTAT "Number of pseudogenes = $m\n";
print "Number of pseudogenes = $m\n";
print OUTSTAT "Number of CDSs = $l\n";
print "Number of CDSs = $l\n";
print OUTSTAT "Number of RNAs = $n\n";
print "Number of RNAs = $n\n";

# Detect and count duplicate gene symbols
if ($gsCheck) {
print "\nDetecting duplicate gene symbols...\n";
for (0..$e) {$tag[$_]=0;}
$n=0;
for ($i=0; $i<$e; $i++) {
	if ($tag[$i]==0) {
		for ($j=$i+1; $j<=$e; $j++) {
			if ($tag[$j]==0) {
				if ($GSB[$j] eq $GSB[$i]) {
					if ($tag[$i]==0) {
						print OUTDATA "\nDuplicate gene symbols:\n\n";
						print OUTDATA "$GSB[$i]\t$GID[$i]\t$pGI[$i]\t$psd[$i]\t$LOC[$i]\t$prd[$i]\n";
						$tag[$i]=1;
					}
					print OUTDATA "$GSB[$j]\t$GID[$j]\t$pGI[$j]\t$psd[$j]\t$LOC[$j]\t$prd[$j]\n";
					$tag[$j]=1;
					$n += 1;
				}
			}
		}
	}
}
print OUTSTAT "Number of duplicate gene symbols = $n\n";
print "Number of duplicate gene symbols = $n\n";
}

# Sort arrays by gene symbol
if ($gbkSort) {
print "\nSorting data by gene symbol...\n";
for ($i=0; $i<$e; $i++) {
	for ($j=$i+1; $j<=$e; $j++) {
		if ($GSB[$j] le $GSB[$i]) {
			splice(@GSB,$i,0,splice(@GSB,$j,1));
			splice(@LOC,$i,0,splice(@LOC,$j,1));
			splice(@GID,$i,0,splice(@GID,$j,1));
			splice(@psd,$i,0,splice(@psd,$j,1));
			splice(@prd,$i,0,splice(@prd,$j,1));
			splice(@pGI,$i,0,splice(@pGI,$j,1));
		}
	}
}}

# Read in oligo data file
print "\nProcessing oligo data file...\n";
@acn=@tag=();
while (<OLGFILE>) {
	my @field=();
	chomp;
	@field = split(/\t/, $_, 8);
	push(@oid,$field[0]); # Get oligo ID
	push(@acn,$field[1]); # Get gene accession
	push(@gsb,$field[2]); # Get gene symbol
	push(@olg,$field[3]); # Get oligo sequence
	push(@loc,$field[6]); # Get gene coordinates
	push(@prt,$field[7]); # Get protein description
	push(@tag,1); # Tag if a gene is included in the sampling population
}
close(OLGFILE);
$m=$#olg;

print OUTSTAT "Number of oligos = ",scalar @olg,"\n";
print "Number of oligos = ",scalar @olg,"\n";

# Check the equality of two loci as their gene symbols match
if ($lcCheck) {
print "\nChecking locus equality...\n";
$n=0;
for ($i=0; $i<=$m; $i++) {
	for ($j=0; $j<=$e; $j++) {
		if ($gsb[$i] eq $GSB[$j]) {
			if ($loc[$i] ne $LOC[$j]) {
				if ($n==0) {
					print OUTDATA "\nUnequal loci with matched gene symbols:\n\n";
					print OUTDATA "GeneSymbol1\tGeneSymbol2\tGene_Loc1\tGene_Loc2\tGene_Product1\tGene_Product2\n\n";
				}
				print OUTDATA "$gsb[$i]\t$GSB[$j]\t$loc[$i]\t$LOC[$j]\t$prt[$i]\t$prd[$j]\n";
				$n += 1;
				last;
			}
		}
	}
}
print OUTSTAT "Number of unequal loci when gene symbols match = $n\n";
print "Number of unequal loci when gene symbols match = $n\n";
}

# Check the equality of two gene symbols as their loci match and integrate them
print "\nChecking gene symbol equality...\n";
$n=0;
for ($i=0; $i<=$m; $i++) {
	for ($j=0; $j<=$e; $j++) {
		if ($gsb[$i] eq $GSB[$j]) {
			$idx[$i]=$j; # mark relational indices
			last;
		} elsif ($loc[$i] eq $LOC[$j]) { # gene symbols unmatch but loci match
			if ($n==0) {
				print OUTDATA "\nUnequal gene symbols with matched loci:\n\n";
				print OUTDATA "GeneSymbol1\tGeneSymbol2\tGene_Loc1\t\tGene_Loc2\t\tGene_Product1\t\tGene_Product2\n\n";
			}
			print OUTDATA "$gsb[$i]\t$GSB[$j]\t$loc[$i]\t$LOC[$j]\t$prt[$i]\t$prd[$j]\n";
			$gsb[$i]=$GSB[$j]; # After this all the equality of two gene symbols is synonymous with that of two loci
			$idx[$i]=$j; # mark relational indices for later join manipulations (w/ gene symbol as a primary key)
			$n += 1;
			last;
		} else { # both gene symbols and loci not matchable
			# '-1' is used to flag there is no relational key or intersectional point between two datasets
			# so any gene symbols (and their entries) other than these pairwise matches will be kept 'as is'.
			$idx[$i]=-1; # mark gene symbol mismatch
		}
	}
}

print OUTSTAT "Number of unequal gene symbols when loci match = $n\n";
print "Number of unequal gene symbols when loci match = $n\n";

print OUTDATA "\n\nGene ignoring/skipping tags and comparisons of attributes between two datasets (Tag=0: Ignored/skipped; Tag=1: Involved in randomized sampling):\n\n";
print OUTDATA "Tag Pseudogene\tAccession#\tGene/GBK_ID\tGeneSymbol1\tGeneSymbol2\tGene_Loc1\tGene_Loc2\tGene_Product1\tGene_Product2\n\n";

print "\nMerging GenBank data with oligo data...\n";
# Join two datasets and skip pseudo-genes and/or RNAs if enabled
for ($i=0; $i<=$m; $i++) {
	if ($idx[$i] > -1) {
		$gid[$i]=$GID[$idx[$i]];
		if ($acn[$i] ne $ACN[$idx[$i]]) {$acn[$i]=$ACN[$idx[$i]];}
		if ($prt[$i] eq "product_name") {$prt[$i]=$prd[$idx[$i]];}
		if ($skiPseu && $psd[$idx[$i]] ne "") {$tag[$i]=0;} # check and skip pseudo-genes
	} else {
		($j)=grep{$c[$_] le $gsb[$i] && $d[$_] ge $gsb[$i]} 0..$#c;
		if (defined($j)) {
			$acn[$i]=$a[$j];
			$gid[$i]=$b[$j];
		} else {
			if ($acn[$i] eq "gene_accession") {$acn[$i]="";}
			$gid[$i]="";
		}
	}
	if ($loc[$i] eq "gene_locus") {$loc[$i]="";} # must meet formatting requirements of gene locus or as a blank otherwise
	if ($prt[$i]=~/$ignItem|^\s*$/i) {$tag[$i]=0;} # check if a gene is hypothetical/putative/pseudo/blank and, if so, ignore it
	if ($skipRNA && $gsb[$i]=~/_R/) {$tag[$i]=0;} # check RNA existence and ignore RNA entries
	if ($idx[$i] > -1) {
		print OUTDATA "$tag[$i]\t$psd[$idx[$i]]\t$acn[$i]\t$gid[$i]\t\t$gsb[$i]\t$GSB[$idx[$i]]\t$loc[$i]\t$LOC[$idx[$i]]\t$prt[$i]\t$prd[$idx[$i]]\n";
	} else {
		print OUTDATA "$tag[$i]\t\t$acn[$i]\t$gid[$i]\t\t$gsb[$i]\t\t$loc[$i]\t\t$prt[$i]\n";
	}
}

#for (0..$#a) {print OUTSTAT "$a[$_]\t$b[$_]\t$c[$_]\t$d[$_]\n";}

# Modify items as per formatting requirements
for ($i=0; $i<=$m; $i++) {
   $prt[$i]=~tr/\|/\//; # > 1 '|' not allowed
 	if ($oid[$i]=~/^(\d*)(\d{4})$/) {
 		$oid[$i]=$1.'_'.$2; # make id a string
 	}
	if ($acn[$i]) {$acn[$i]="ref|$acn[$i]";}
	if ($gid[$i]) {$gid[$i]="gi|$gid[$i]";}
	if ($loc[$i]=~/(\d+)\.\.(\d+)\(\+\)/) {
		$loc[$i]="$s[2]chr1:$1-$2";
	} elsif ($loc[$i]=~/(\d+)\.\.(\d+)\(-\)/) {
		$loc[$i]="$s[2]chr1:$2-$1";
	} elsif ($loc[$i]!~/^\s*$/) {
		warn "$loc[$i]: Invalid coordinates!\n";
	}
}

# Generate biological oligo group and randomized replicate oligo group
print "Generating biological and replicate oligos...\n";
print OUTDATA "\n\n\nRandomly-sampled oligo entries and randomized numbers:\n\n";
print OUTDATA "Oligo_ID\tGene_Symbol\tRandom_Number\n\n";

$i=0;
$n=@olg;
$rnd[$i]=int(rand($n));
while ($i==0) {
	if ($tag[$rnd[$i]]==1) {
		print REPFILE "$oid[$rnd[$i]]\t$olg[$rnd[$i]]\t$acn[$rnd[$i]]\t$gid[$rnd[$i]]\t$gsb[$rnd[$i]]\t$prt[$rnd[$i]]\t$loc[$rnd[$i]]\n";
		print OUTDATA "$oid[$rnd[$i]]\t$gsb[$rnd[$i]]\t$rnd[$i]\n";
		$tag[$rnd[$i]]=2;
		$i += 1;
	} else {
		$rnd[$i]=int(rand($n));
	}
}
while ($i < $numReps) {
	$rnd[$i]=int(rand($n));
	for ($j=$i-1; $j>=0; $j--) {
		last if ($rnd[$i]==$rnd[$j]);
	}
	if ($j < 0) {
		if ($tag[$rnd[$i]]==1) {
			print REPFILE "$oid[$rnd[$i]]\t$olg[$rnd[$i]]\t$acn[$rnd[$i]]\t$gid[$rnd[$i]]\t$gsb[$rnd[$i]]\t$prt[$rnd[$i]]\t$loc[$rnd[$i]]\n";
			print OUTDATA "$oid[$rnd[$i]]\t$gsb[$rnd[$i]]\t$rnd[$i]\n";
			$tag[$rnd[$i]]=2;
			$i += 1;
		}
	}
}

print OUTSTAT "Number of replicate oligos = $i\n";
print "\nNumber of replicate oligos = $i\n";

$n=0;
for ($i=0; $i<=$m; $i++) {
	if ($tag[$i] != 2) {
		print BIOFILE "$oid[$i]\t$olg[$i]\t$acn[$i]\t$gid[$i]\t$gsb[$i]\t$prt[$i]\t$loc[$i]\n";
		$n += 1;
	}
}

print OUTSTAT "Number of biological oligos = $n\n";
print "Number of biological oligos = $n\n\n";

# Output GenBank data for info
print OUTDATA "\n\n\nGenBank data extracted from GenBank file(s):\n\n";
print OUTDATA "Accession#\tGene_Symbol\tGene_ID\tProtein_GI\tPseudogene\tGene_Loc\tGene_Product\n\n";
for ($i=0; $i<=$e; $i++) {
	print OUTDATA "$ACN[$i]\t$GSB[$i]\t$GID[$i]\t$pGI[$i]\t$psd[$i]\t$LOC[$i]\t$prd[$i]\n";
}

close(BIOFILE);
close(REPFILE);
close(OUTDATA);
close(OUTSTAT);
print "Done!\n\n";
